/**
 * @file
 * @ingroup <<<GROUP>>>
 * @brief   <<<BRIEF>>>
 *
 *          This code is Autogenerated from '<<<PYIFGENNAME>>>' with the MIT License.
 *          As such, please only hand-code within 'USER' tags.
 *
 * @author  <<<AUTHOR>>>
 */

<<<NOT_FORWARD_DECLARABLE_HEADER_INCLUDES>>>

/// {{{USER_HEADER_INCLUDES}}}
/// {{{USER_HEADER_INCLUDES}}}

<<<NESTED_NAMESPACE_BEGIN>>>

    /// {{{USER_LOCALS}}}
    /// {{{USER_LOCALS}}}

    <<<CLASS_DOCUMENTATION>>>
    <<<PACK_BEGIN>>>
    public struct <<<CLASSNAME>>>
    {
        <<<STRUCT_EXPAND>>>
    };

<<<NESTED_NAMESPACE_END>>>