/**
 * @file
 * @ingroup <<<GROUP>>>
 * @brief   <<<BRIEF>>>
 *
 *          This code is Autogenerated from '<<<PYIFGENNAME>>>' with the MIT License.
 *          As such, please only hand-code within 'USER' tags.
 *
 * @author  <<<AUTHOR>>>
 */

<<<NOT_FORWARD_DECLARABLE_HEADER_INCLUDES>>>

/// {{{USER_HEADER_INCLUDES}}}
/// {{{USER_HEADER_INCLUDES}}}

<<<NESTED_NAMESPACE_BEGIN>>>

    /// {{{USER_LOCALS}}}
    /// {{{USER_LOCALS}}}

    <<<CLASS_DOCUMENTATION>>>
    public class <<<DLL_EXPORT>>><<<CLASSNAME>>><<<CLASS_INHERITENCE_HIERARCHY>>>
    {
        <<<CONSTRUCTORS_DECLARE>>>

        <<<PUBLIC_ATTRIBUTES_DECLARE>>>
        <<<PUBLIC_ASSOCIATIONS_DECLARE>>>
        <<<ATTRIBUTES_ASSOCIATIONS_GETTER_SETTER_DECLARE>>>
        <<<PUBLIC_OPERATIONS_DECLARE>>>

        /// {{{USER_PUBLIC}}}
        /// {{{USER_PUBLIC}}}

        <<<PROTECTED_ATTRIBUTES_DECLARE>>>
        <<<PROTECTED_ASSOCIATIONS_DECLARE>>>
        <<<PROTECTED_OPERATIONS_DECLARE>>>

        /// {{{USER_PROTECTED}}}
        /// {{{USER_PROTECTED}}}

        <<<PRIVATE_ATTRIBUTES_DECLARE>>>
        <<<PRIVATE_ASSOCIATIONS_DECLARE>>>
        <<<PRIVATE_OPERATIONS_DECLARE>>>

        /// {{{USER_PRIVATE}}}
        /// {{{USER_PRIVATE}}}
    }

<<<NESTED_NAMESPACE_END>>>