/**
 * @file
 * @ingroup <<<GROUP>>>
 * @brief   <<<BRIEF>>>
 *
 *          This code is Autogenerated from '<<<PYIFGENNAME>>>.py' with the MIT License.
 *          As such, please only hand-code within 'USER' tags.
 *
 * @author  <<<AUTHOR>>>
 */

#ifndef PROTOCOL_HPP
#define PROTOCOL_HPP

// {{{USER_INCLUDES}}}
// {{{USER_INCLUDES}}}

#if defined(_MSC_VER)
#define __attribute__(x)
#pragma pack(push,1)
#elif defined(__GNUC__)
#elif defined(__clang__)
#endif  // #if defined(_MSC_VER)

/******************************************************
* @brief Protocol struct definitions
******************************************************/

<<<PER_PROTOMSG_BEGIN>>>
struct <<<PROTOMSGNAME>>>
{
<<<MEMBERSDECLARE>>>
};
<<<PER_PROTOMSG_END>>>


#if defined(_MSC_VER)
#pragma pack(pop)
#elif defined(__GNUC__) || defined(__clang__)
#endif // #if defined(_MSC_VER)

#endif // #ifndef PROTOCOL_HPP
