/**
 * @file
 * @ingroup <<<GROUP>>>
 * @brief   <<<BRIEF>>>
 *
 *          This code is Autogenerated from '<<<PYIFGENNAME>>>' with the MIT License.
 *          As such, please only hand-code within 'USER' tags.
 *
 * @author  <<<AUTHOR>>>
 */
/// {{{USER_PCH}}}
/// {{{USER_PCH}}}
#include "<<<CLASSNAME>>>.h"

<<<FORWARD_DECLARABLE_HEADER_INCLUDES>>>

/// {{{USER_HEADER_INCLUDES}}}
/// {{{USER_HEADER_INCLUDES}}}

<<<NESTED_NAMESPACE_BEGIN>>>
    <<<STATIC_ATTRIBUTES_DECLARE>>>
    <<<STATIC_ASSOCIATIONS_DECLARE>>>

    <<<CLASSNAME>>>::~<<<CLASSNAME>>>(){
        /// {{{USER_DESTRUCTOR_<<<CLASSNAME>>>}}}
        /// {{{USER_DESTRUCTOR_<<<CLASSNAME>>>}}}
    }
    <<<CONSTRUCTORS_IMPLEMENTATION>>>
    <<<ATTRIBUTES_ASSOCIATIONS_GETTER_SETTER_IMPLEMENTATION>>>
    <<<OPERATIONS_IMPLEMENTATION>>>

    /// {{{USER_PUBLIC_PROTECTED_PRIVATE}}}
    /// {{{USER_PUBLIC_PROTECTED_PRIVATE}}}

<<<NESTED_NAMESPACE_END>>>