/**
 * @file
 * @ingroup <<<GROUP>>>
 * @brief   <<<BRIEF>>>
 *
 *          This code is Autogenerated from '<<<PYIFGENNAME>>>' with the MIT License.
 *          As such, please only hand-code within 'USER' tags.
 *
 * @author  <<<AUTHOR>>>
 */
#include "<<<CLASSNAME>>>Transmitter.h"
namespace <<<NAMESPACE>>>
{
    /** Transmitters.
    */
    <<<PER_MSG_BEGIN>>>
    bool <<<CLASSNAME>>>Transmitter::Transmit<<<MSGNAME>>>(const <<<MSGNAME>>>& data, int8 retries) const
    {
        bool ok = false;
        for (; retries >= 0 && !ok && (connection != nullptr); retries--) {
            ok = ok || connection->SendData(reinterpret_cast<const uint8*>(&data), sizeof(<<<MSGNAME>>>));
        }
        return ok;
    }
    <<<PER_MSG_END>>>

    void <<<CLASSNAME>>>Transmitter::TestSendAll()
    {
        <<<PER_MSG_BEGIN>>>
        Transmit<<<MSGNAME>>>(Create<<<MSGNAME>>>());
        <<<PER_MSG_END>>>   
    }
} // namespace <<<NAMESPACE>>>
